# -*- coding: UTF-8 -*-
#
#  Copyright (C) 2004 by Shyouzou Sugitani <shy@users.sourceforge.jp>
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License (version 2) as
#  published by the Free Software Foundation.  It is distributed in the
#  hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#  PURPOSE.  See the GNU General Public License for more details.
#

# TODO:
# - 表示位置の調整
# - アニメーション
# - イベント送信の引数
# - サーフェス移動への追従
# - ゴースト終了時の処理
# - 透明度の設定

import os
import sys

if os.environ.has_key("DISPLAY"):
    import gtk
    import pango

import ninix.pix

class Menu:
    def __init__(self, kinoko, accelgroup):
        self.__kinoko = kinoko
        self.__menu_list = {
            'settings': [_('Settings...(_O)'), None, ('/settings', None, self.__kinoko.edit_preferences, 0, '')],
            'skin': [_('Skin(_K)'), None, ('/skin', None, None, 0, '<Branch>')],
            'exit':     [_('Exit(_Q)'), None, ('/exit', None, self.__kinoko.close, 0, '')],
            }
        self.__skin_list = None
        menu_items = (
            self.__menu_list['settings'][-1],
            self.__menu_list['skin'][-1],
            ('/Separator', None, None, 0, '<Separator>'),
            self.__menu_list['exit'][-1],
            )
        self.__item_factory = gtk.ItemFactory(gtk.Menu, '<main>', accelgroup)
        self.__item_factory.create_items(menu_items)
        self.__popup_menu = self.__item_factory.get_widget('<main>')
        for key in self.__menu_list.keys():
            path = self.__menu_list[key][-1][0]
            self.__menu_list[key][1] = self.__item_factory.get_item(path)
    def popup(self, button):
        skin_list = self.__kinoko.get_skin_list()
        self.__set_skin_menu(skin_list)
        try:
            self.__popup_menu.popup(None, None, None, button, gtk.get_current_event_time())
        except: # pygtk <= 2.0.0
            self.__popup_menu.popup(None, None, None, button, 0)
    def __set_skin_menu(self, list): ## FIXME
        key = 'skin'
        if list:
            menu = gtk.Menu()
            for skin in list:
                item = gtk.MenuItem(skin['title'])
                item.connect("activate", self.__kinoko.select_skin, (skin))
                menu.add(item)
                item.show()
            self.__menu_list[key][1].set_submenu(menu)
            menu.show()
            self.__menu_list[key][1].show()
        else:
            self.__menu_list[key][1].hide()

class Kinoko:
    def __init__(self, skin_list):
        self.skin_list = skin_list
    def edit_preferences(self):
        pass
    def load(self, data, target):
        self.data = data
        self.target = target
        self.accelgroup = gtk.AccelGroup()
        self.skin = Skin(self.data, self.accelgroup, self)
        if self.skin == None:
            return 0
        else:
            self.target.get_sakura().notify_event('OnKinokoObjectCreate', self.data['title'])
    def close(self, widget, args):
        self.skin.destroy()
        self.target.get_sakura().notify_event('OnKinokoObjectDestroy')
    def get_skin_list(self):
        return self.skin_list
    def select_skin(self, widget, args):
        self.target.get_sakura().notify_event('OnKinokoObjectChanging')
        self.skin.destroy()
        self.data = args
        self.skin = Skin(self.data, self.accelgroup, self)
        if self.skin == None:
            return 0
        else:
            self.target.get_sakura().notify_event('OnKinokoObjectChanged', self.data['title'])

class Skin:
    def __init__(self, data, accelgroup, kinoko):
        self.data = data
        self.accelgroup = accelgroup
        self.kinoko = kinoko
        self.__menu = Menu(self.kinoko, self.accelgroup)
        self.window = gtk.Window()
        ##self.window.set_title('surface.' + name)
        self.window.set_decorated(gtk.FALSE)
        self.window.set_resizable(gtk.FALSE)
        self.window.connect("delete_event", self.delete)
        self.window.connect("button_press_event", self.button_press)
        self.window.connect("button_release_event", self.button_release)
        self.window.connect("motion_notify_event", self.motion_notify)
        self.window.connect("leave_notify_event", self.leave_notify)
        self.window.set_events(gtk.gdk.BUTTON_PRESS_MASK|
                               gtk.gdk.BUTTON_RELEASE_MASK|
                               gtk.gdk.POINTER_MOTION_MASK|
                               gtk.gdk.LEAVE_NOTIFY_MASK)
        self.window.realize()
        self.window.add_accel_group(self.accelgroup)
        scrn_w = gtk.gdk.screen_width()
        scrn_h = gtk.gdk.screen_height()
        path = os.path.join(self.data['dir'], self.data['base'])
        try:
            image, mask = ninix.pix.create_pixmap_from_file(path)
        except: ## FIXME
            self.destroy()
            return None
        image, mask = ninix.pix.create_pixmap_from_file(path)
        ## FIXME
        w, h = image.get_size()
        target_x, target_y = self.kinoko.target.get_position(0)
        target_w, target_h = self.kinoko.target.get_surface_size(0)
        self.x = target_x + self.data['offsetx'] + target_w / 2 - w / 2
        self.y = target_y + self.data['offsety'] - h
        self.window.move(self.x, self.y)
        self.mask_pixmap = mask
        self.window.shape_combine_mask(self.mask_pixmap, 0, 0)
        self.darea = gtk.DrawingArea()
        self.darea.set_events(gtk.gdk.EXPOSURE_MASK)
        self.darea.connect("expose_event", self.redraw)
        self.darea.set_size_request(w, h)
        self.darea.show()
        self.window.add(self.darea)
        self.darea.realize()
        self.darea.window.set_back_pixmap(image, gtk.FALSE)
        self.window.show()
    def redraw(self, widget, event):
        pass
    def delete(self, widget, event):
        self.kinoko.close()
    def destroy(self): ## FIXME
        self.window.destroy()
    def button_press(self, widget, event): ## FIXME
        x = int(event.x)
        y = int(event.y)
        self.x_root = event.x_root
        self.y_root = event.y_root
        if event.type == gtk.gdk.BUTTON_PRESS:
            click = 1
        else:
            click = 2
        button = event.button
        if button == 3 and click == 1:
            self.__menu.popup(button)
        return gtk.TRUE
    def button_release(self,  widget, event): ## FIXME
        pass
    def motion_notify(self,  widget, event): ## FIXME
        pass
    def leave_notify(self,  widget, event): ## FIXME
        pass
