# -*- coding: UTF-8 -*-
#
#  Copyright (C) 2004 by Shyouzou Sugitani <shy@users.sourceforge.jp>
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License (version 2) as
#  published by the Free Software Foundation.  It is distributed in the
#  hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#  PURPOSE.  See the GNU General Public License for more details.
#

# TODO:
# - 「きのこ」へのステータス送信.
# - 「きのこ」の情報の参照.
# - SERIKO/1.2ベースのアニメーション
# - katochan.txt
# - balloon.txt
# - surface[0/1/2]a.txt
# - install.txt
# - profile.txt
# - 自爆イベント
# - headrect.txt
# - speak.txt
# - omni.txt
# - サーフェス倍率対応.
# - katochan が無い場合の処理.(本体の方のpopup menuなども含めて)

import os
import sys

if os.environ.has_key("DISPLAY"):
    import gtk
    import pango

import ninix.pix

class Menu:
    def __init__(self, nekoninni, accelgroup):
        self.__nekoninni = nekoninni
        self.__menu_list = {
            'settings': [_('Settings...(_O)'), None, ('/settings', None, self.__nekoninni.edit_preferences, 0, '')],
            'katochan': [_('Katochan(_K)'), None, ('/katochan', None, None, 0, '<Branch>')],
            'exit':     [_('Exit(_Q)'), None, ('/exit', None, self.__nekoninni.close, 0, '')],
            }
        self.__katochan_list = None
        menu_items = (
            self.__menu_list['settings'][-1],
            self.__menu_list['katochan'][-1],
            ('/Separator', None, None, 0, '<Separator>'),
            self.__menu_list['exit'][-1],
            )
        self.__item_factory = gtk.ItemFactory(gtk.Menu, '<main>', accelgroup)
        self.__item_factory.create_items(menu_items)
        self.__popup_menu = self.__item_factory.get_widget('<main>')
        for key in self.__menu_list.keys():
            path = self.__menu_list[key][-1][0]
            self.__menu_list[key][1] = self.__item_factory.get_item(path)
    def popup(self, button):
        katochan_list = self.__nekoninni.get_katochan_list()
        self.__set_katochan_menu(katochan_list)
        try:
            self.__popup_menu.popup(None, None, None, button, gtk.get_current_event_time())
        except: # pygtk <= 2.0.0
            self.__popup_menu.popup(None, None, None, button, 0)
    def __set_katochan_menu(self, list): ## FIXME
        key = 'katochan'
        if list:
            menu = gtk.Menu()
            for katochan in list:
                item = gtk.MenuItem(katochan)
                item.connect("activate", self.__nekoninni.select_katochan, (katochan))
                menu.add(item)
                item.show()
            self.__menu_list[key][1].set_submenu(menu)
            menu.show()
            self.__menu_list[key][1].show()
        else:
            self.__menu_list[key][1].hide()

class Nekoninni:
    EVENT_LIST = ['Emerge', 'Hit', 'Drop', 'Vanish'] # OnNekodorifObject%s
    def __init__(self):
        self.mode = 0 # 0: SEND SSTP1.1, 1: SHIORI/2.2
    def load(self, dir, katochan, target):
        if len(katochan) == 0:
            return 0
        self.dir = dir
        self.target = target
        self.accelgroup = gtk.AccelGroup()
        self.skin = Skin(self.dir, self.accelgroup, self)
        if self.skin == None:
            return 0
        self.katochan_list = katochan
        self.katochan = None
        self.launch_katochan(self.katochan_list[0])
    def select_katochan(self, widget, args):
        self.launch_katochan(args[0])
    def drop_katochan(self):
        self.katochan.drop()
    def launch_katochan(self, katochan):
        if self.katochan:
            self.katochan.destroy()
        self.katochan = Katochan()
        self.katochan.load(katochan)
        ## FIXME
        name = self.katochan.get_name()
        ghost_name = self.katochan.get_ghost_name()
        category = self.katochan.get_category()
        kinoko_flag = self.katochan.get_kinoko_flag()
        target = self.katochan.get_target()
        ## FIXME
        sakura = self.target.get_sakura()
        sakura.notify_event('OnNekodorifObjectEmerge', name, ghost_name, category, kinoko_flag, target)
    def edit_preferences(self, widget, args):
        pass
    def close(self, widget, args):
        ##self.katochan.destroy()
        self.skin.destroy()
        ##self.balloon.destroy()
    def get_katochan_list(self): ## FIXME
        return []

class Skin:
    def __init__(self, dir, accelgroup, nekoninni):
        self.dir = dir
        self.accelgroup = accelgroup
        self.nekoninni = nekoninni
        self.__menu = Menu(self.nekoninni, self.accelgroup)
        self.window = gtk.Window()
        ##self.window.set_title('surface.' + name)
        self.window.set_decorated(gtk.FALSE)
        self.window.set_resizable(gtk.FALSE)
        self.window.connect("delete_event", self.delete)
        self.window.connect("button_press_event", self.button_press)
        self.window.connect("button_release_event", self.button_release)
        self.window.connect("motion_notify_event", self.motion_notify)
        self.window.connect("leave_notify_event", self.leave_notify)
        self.window.set_events(gtk.gdk.BUTTON_PRESS_MASK|
                               gtk.gdk.BUTTON_RELEASE_MASK|
                               gtk.gdk.POINTER_MOTION_MASK|
                               gtk.gdk.LEAVE_NOTIFY_MASK)
        self.window.realize()
        self.window.add_accel_group(self.accelgroup)
        scrn_w = gtk.gdk.screen_width()
        scrn_h = gtk.gdk.screen_height()
        path = os.path.join(self.dir, 'surface0.png')
        try:
            image, mask = ninix.pix.create_pixmap_from_file(path)
        except: ## FIXME
            self.destroy()
            return None
        image, mask = ninix.pix.create_pixmap_from_file(path)
        w, h = image.get_size()
        self.x = 0
        self.y = scrn_h - h
        self.window.move(self.x, self.y)
        self.mask_pixmap = mask
        self.window.shape_combine_mask(self.mask_pixmap, 0, 0)
        self.darea = gtk.DrawingArea()
        self.darea.set_events(gtk.gdk.EXPOSURE_MASK)
        self.darea.connect("expose_event", self.redraw)
        self.darea.set_size_request(w, h)
        self.darea.show()
        self.window.add(self.darea)
        self.darea.realize()
        self.darea.window.set_back_pixmap(image, gtk.FALSE)
        self.window.show()
    def redraw(self, widget, event):
        pass
    def delete(self,  widget, event):
        self.nikoninni.close()
    def destroy(self): ## FIXME
        self.window.destroy()
    def button_press(self,  widget, event): ## FIXME
        x = int(event.x)
        y = int(event.y)
        self.x_root = event.x_root
        self.y_root = event.y_root
        if event.type == gtk.gdk.BUTTON_PRESS:
            click = 1
        else:
            click = 2
        button = event.button
        if button == 3 and click == 1:
            self.__menu.popup(button)
        elif button == 1 and click == 2:
            self.start() ## FIXME
            self.nekoninni.drop_katochan()
        return gtk.TRUE
    def start(self): ## FIXME
        path = os.path.join(self.dir, 'surface1.png')
        try:
            image, mask = ninix.pix.create_pixmap_from_file(path)
        except: ## FIXME
            self.destroy()
            return None
        image, mask = ninix.pix.create_pixmap_from_file(path)
        w, h = image.get_size()
        #self.x = 0 ## FIXME
        #self.y = scrn_h - h ## FIXME
        #self.window.move(self.x, self.y)
        self.mask_pixmap = mask
        self.window.shape_combine_mask(self.mask_pixmap, 0, 0)
        self.darea.set_size_request(w, h)
        self.darea.window.set_back_pixmap(image, gtk.FALSE)
    def button_release(self,  widget, event): ## FIXME
        pass
    def motion_notify(self,  widget, event): ## FIXME
        pass
    def leave_notify(self,  widget, event): ## FIXME
        pass

class Balloon:
    def __init__(self):
        pass
    def destroy(self): ## FIXME
        pass

class Katochan:
    CATEGORY_LIST = ['pain',      # 痛い
                     'stab',      # 刺さる
                     'surprise',  # びっくり
                     'hate',      # 嫌い、気持ち悪い
                     'huge',      # 巨大
                     'love',      # 好き、うれしい
                     'elegant',   # 風流、優雅
                     'pretty',    # かわいい
                     'food',      # 食品
                     'reference', # 見る／読むもの
                     'other'      # 上記カテゴリに当てはまらないもの
                     ]
    def __init__(self):
        self.category = {}
        self.category['major'] = self.CATEGORY_LIST[-1]
        self.category['minor'] = []
        self.target = None # for
        pass
    def destroy(self): ## FIXME
        pass
